/**
 * Redistribution and use of this software and associated documentation
 * ("Software"), with or without modification, are permitted provided
 * that the following conditions are met:
 *
 * 1. Redistributions of source code must retain copyright
 *    statements and notices.  Redistributions must also contain a
 *    copy of this document.
 *
 * 2. Redistributions in binary form must reproduce the
 *    above copyright notice, this list of conditions and the
 *    following disclaimer in the documentation and/or other
 *    materials provided with the distribution.
 *
 * 3. The name "Exolab" must not be used to endorse or promote
 *    products derived from this Software without prior written
 *    permission of Exoffice Technologies.  For written permission,
 *    please contact info@exolab.org.
 *
 * 4. Products derived from this Software may not be called "Exolab"
 *    nor may "Exolab" appear in their names without prior written
 *    permission of Exoffice Technologies. Exolab is a registered
 *    trademark of Exoffice Technologies.
 *
 * 5. Due credit should be given to the Exolab Project
 *    (http://www.exolab.org/).
 *
 * THIS SOFTWARE IS PROVIDED BY EXOFFICE TECHNOLOGIES AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL
 * EXOFFICE TECHNOLOGIES OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Copyright 1999 (C) Exoffice Technologies Inc. All Rights Reserved.
 *
 * $Id: EjbRefMetaData.java,v 1.2 2002/02/25 20:30:50 dblevins Exp $
 */
package org.openejb.alt.assembler.modern.jar.ejb11;

/**
 * Metadata for an EJB's reference to another bean.  This includes the
 * target beans's identifier (if deployed in the same application) and
 * characteristics.  If the bean is not deployed in the same application,
 * you have to use the server-specific metadata to locate it.
 *
 * @author Aaron Mulder (ammulder@alumni.princeton.edu)
 * @version $Revision: 1.2 $
 */
public class EjbRefMetaData {
    private String description;
    private String refName;
    private boolean refIsSession;
    private String homeInterfaceName;
    private String remoteInterfaceName;
    private String ejbLink;

    public EjbRefMetaData() {
    }
    public EjbRefMetaData(EjbRefMetaData source) {
        description = source.description;
        refName = source.refName;
        refIsSession = source.refIsSession;
        homeInterfaceName = source.homeInterfaceName;
        remoteInterfaceName = source.remoteInterfaceName;
        ejbLink = source.ejbLink;
    }

    public void setDescription(String desc) {description = desc;}
    public String getDescription() {return description;}
    public void setRefName(String name) {refName = name;}
    public String getRefName() {return refName;}
    public void setRefTypeSession(boolean isSession) {refIsSession = isSession;}
    public boolean isRefTypeSession() {return refIsSession;}
    public void setHomeInterfaceName(String name) {homeInterfaceName = name;}
    public String getHomeInterfaceName() {return homeInterfaceName;}
    public void setRemoteInterfaceName(String name) {remoteInterfaceName = name;}
    public String getRemoteInterfaceName() {return remoteInterfaceName;}
    public void setEjbLink(String linkedEjbName) {ejbLink = linkedEjbName;}
    public String getEjbLink() {return ejbLink;}

    public String toString() {
        return getRefName();
    }
}
